<?php

namespace Drupal\terms_export_import\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\Exception\FileException;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Validation\FileValidatorInterface;
use Drupal\terms_export_import\Service\TermsExportImportService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class TermsImportForm. Provides the import form for terms.
 */
class TermsImportForm extends FormBase {

  /**
   * Constructs a new TermsImportForm instance with injected dependencies.
   *
   * @param \Drupal\terms_export_import\Service\TermsExportImportService $exportImportService
   *   The terms export/import service.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system service.
   * @param \Drupal\file\Validation\FileValidatorInterface $fileValidator
   *   The file validator service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   */
  public function __construct(
    protected TermsExportImportService $exportImportService,
    protected FileSystemInterface $fileSystem,
    protected FileValidatorInterface $fileValidator,
    protected RequestStack $requestStack,
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): TermsImportForm {
    return new static(
      $container->get('terms_export_import.service'),
      $container->get('file_system'),
      $container->get('file.validator'),
      $container->get('request_stack'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'terms_import_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['import_file'] = [
      '#type'        => 'file',
      '#title'       => $this->t('CSV file'),
      '#description' => $this->t('Upload a CSV file with columns: name,description,parent,weight'),
      '#required'    => TRUE,
    ];

    $form['vocabulary'] = [
      '#type'        => 'entity_autocomplete',
      '#title'       => $this->t('Vocabulary'),
      '#target_type' => 'taxonomy_vocabulary',
      '#required'    => TRUE,
      '#description' => $this->t('Select the vocabulary to import into.'),
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type'  => 'submit',
      '#value' => $this->t('Import CSV'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $uploaded_files = $this->requestStack->getCurrentRequest()->files->get('files', []);

    /** @var \Symfony\Component\HttpFoundation\File\UploadedFile|null $uploaded_file */
    $uploaded_file = $uploaded_files['import_file'] ?? NULL;

    if (!$uploaded_file || !$uploaded_file->isValid()) {
      $form_state->setErrorByName('import_file', $this->t('Please upload a valid CSV file.'));
      return;
    }

    $filename = $uploaded_file->getClientOriginalName();
    $temp_directory = 'temporary://taxonomy_import_export';
    $this->fileSystem->prepareDirectory(
      $temp_directory,
      FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS
    );
    $temp_uri = $temp_directory . '/' . $filename;

    $realpath = $this->fileSystem->realpath($temp_uri);
    if ($realpath && file_exists($realpath)) {
      $this->fileSystem->delete($temp_uri);
    }

    file_put_contents($this->fileSystem->realpath($temp_uri), file_get_contents($uploaded_file->getRealPath()));

    $file_entity = $this->entityTypeManager->getStorage('file')->create([
      'uri'    => $temp_uri,
      'status' => 0,
    ]);
    $file_entity->setFilename($filename);
    $file_entity->save();

    $validators = [
      'FileExtension' => ['extensions' => 'csv'],
    ];

    $violations = $this->fileValidator->validate($file_entity, $validators);

    if ($violations->count() > 0) {
      $this->fileSystem->delete($temp_uri);
      $file_entity->delete();

      /** @var \Symfony\Component\Validator\ConstraintViolationInterface $first_violation */
      $first_violation = $violations[0];
      $message = $this->t('File validation error: @msg', ['@msg' => $first_violation->getMessage()]);
      $form_state->setErrorByName('import_file', $message);
      return;
    }

    $form_state->setValue('temp_file_id', $file_entity->id());
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $fid = $form_state->getValue('temp_file_id');
    if (!$fid) {
      $this->messenger()->addError($this->t('No valid file to import.'));
      return;
    }

    /** @var \Drupal\file\FileInterface $file */
    $file = $this->entityTypeManager->getStorage('file')->load($fid);
    if (!$file) {
      $this->messenger()->addError($this->t('Could not load the file entity.'));
      return;
    }

    $file->setPermanent();

    $destination_directory = 'public://taxonomy_import_export';
    $this->fileSystem->prepareDirectory(
      $destination_directory,
      FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS
    );
    $destination = $destination_directory . '/' . $file->getFilename();

    $destination_realpath = $this->fileSystem->realpath($destination);
    if ($destination_realpath && file_exists($destination_realpath)) {
      $this->fileSystem->delete($destination);
    }

    try {
      $this->fileSystem->move($file->getFileUri(), $destination);
    }
    catch (FileException $e) {
      $this->messenger()->addError($this->t('Could not move uploaded file: @message', [
        '@message' => $e->getMessage(),
      ]));
      return;
    }

    $file->setFileUri($destination);
    $file->save();

    $csv_string = file_get_contents($file->getFileUri());
    $vocabulary_id = $form_state->getValue('vocabulary');

    $vocab = $this->entityTypeManager->getStorage('taxonomy_vocabulary')->load($vocabulary_id);
    if (!$vocab) {
      $this->messenger()->addError($this->t('Invalid vocabulary selected.'));
      return;
    }

    $terms_data = $this->exportImportService->parseCsv($csv_string);
    $this->exportImportService->importTerms($vocab->id(), $terms_data);

    $this->messenger()->addStatus($this->t('Taxonomy terms have been successfully imported.'));
  }

}
